const mongoose = require('mongoose');

const Cliente = mongoose.model('clientes');

module.exports = app => {
  app.get('/api/clientes', async (req, res) => {
    console.info('Obteniendo clientes');
    const clientes = await Cliente.find({});
    res.send(clientes);
  });

  app.get('/api/clientes/:id', async (req, res) => {
    try {
      const id = req.params.id;
      const cliente = await Cliente.findById(id);

      if (cliente) {
        res.send(cliente);
      } else {
        res
          .status(404)
          .send({ mensaje: `El cliente con id '${id}' no ha sido encontrado.` });
      }
    } catch (e) {
      res.status(500).send({ mensaje: `Error en el servidor.\n\n${e}` });
    }
  });
  app.post('/api/clientes',function(req,res){
    Cliente.create(req.body, function(err, clientes){
        if(err){
            res.send("Error al agregar cliente");
        }else{
            res.json(clientes);
        }
    });
});
  app.put('/api/clientes/:id', async (req, res) => {
    const id = req.params.id;

    const datosCliente = req.body || {};
 
//Cuando no le pone status, siempre devuelve el 200.
    try {
      let cliente = await Cliente.findByIdAndUpdate({ _id: id }, datosCliente, {
        new: true
      });

      if (!cliente) {
        res.status(404).send({
          mensaje: `Error cuando se modificaban los datos del cliente con id ${id}.\n\n${e}`
        });
      } else {
        res.status(200).send(cliente);
      }
    } catch (err) {
      if (err.name === 'MongoError') {
        res.status(409).send({ mensaje: err.message });
      }
      res.status(500).send({
        mensaje: `Error desconocido.\n\n Error desconocido cuando se actualizaba el cliente id='${id}'`
      });
    }
  });

  app.delete('/api/clientes/:id', async (req, res) => {
    const id = req.params.id;

    try {
      let cliente = await Cliente.findByIdAndRemove({ _id: id });

      if (!cliente) {
        return res.status(404).send({ mensaje: 'Cliente no encontrado' });
      } else {
        return res.status(204).send({ mensaje: 'Registro eliminado' }); // 204 do not use content
      }
    } catch (err) {
      return res.status(500).send({
        mensaje: `Error desconocido cuando se borraba cliente con id '${id}'.`
      });
    }
  });
  

  app.put('/api/clientes/:id/cambiardireccion', async (req, res) => {
    const id = req.params.id;

    const cliente = await Cliente.findOne({ _id: id });

    if (!cliente) {
      return res.status(404).send({
        mensaje: `El cliente con id ${id} no ha sido encontrado.\n\n${e}`
      });
    }

    if (cliente) {
      cliente.direccion = new Direccion(req.body);
      cliente.save();
      res.status(200).send(cliente);
    }
  });

  app.get('/api/cliente/consulta', async (req, res) => {
    try {
      var regExpTerm = new RegExp(req.query.consulta, 'i');
      var regExpSearch = [
        { nombre: { $regex: regExpTerm } },
        { email: { $regex: regExpTerm } },
        {cuil: { $regex: regExpTerm }},
        {telefono: { $regex: regExpTerm }},
        {direccion: { $regex: regExpTerm }},
      ];
      const clientes = await Cliente.find({ $or: regExpSearch });

      res.status(200).send(clientes);
    } catch (e) {
      res.status(500).send({ mensaje: e });
    }
  });
};
