const mongoose = require('mongoose');

const Factura = mongoose.model('facturas');

const Cliente = mongoose.model('clientes');

module.exports = app => {


    app.get('/api/facturas',function(req,res){
        Factura.find({}, function(err, facturas) {
            Cliente.populate(facturas, {path: "clientes"},function(err, facturas){
                res.status(200).send(facturas);
            }); 
        });
    });

    app.get('/api/facturas/clientes/:nombre',function(err,facturas){

        const{nombre} = req.params;

        Factura.find({cliente:nombre,estado:'false'},function(err,facturas){
                Cliente.populate(facturas,{path:"clientes"},function(err,facturas){
                    res.status(200).send(facturas);
                });
        });


    });
   
    app.get("/api/facturas/:id", async (req, res) => {

        try {
            const id = req.params.id;
            const factura = await Factura.findById(id);

            if (factura) {
                res.send(factura);
            } else {
                res.status(404).send({ message: `La factura con id: '${id}' no ha podido ser encontrada. ` });
            }
        } catch (e) {
            res.status(500).send({ message: `Error en el servidor.\n\n${e}` });
        }

    });


    app.post('/api/facturas',function(req,res){
        Factura.create(req.body, function(err, facturas){
            if(err){
                res.send("Error al agregar la factura");
            }else{
                res.json(facturas);
            }
        });
    });


    app.put("/api/facturas/:id", async (req, res) => {

        const id = req.params.id;

        const datosFactura = req.body || {};
        delete datosFactura.createdAt;
        datosFactura.updatedAt = new Date();

        try {
            let factura = await Factura.findOneAndUpdate({ _id: id }, datosFactura, { new: true });

            if (!factura) {
                res.status(404).send({ message: `Error cuando se actualizo factura con id: ${id}.\n\n${e}` })
            } else {
                res.status(200).send(factura);
            }
        } catch (err) {
            if (err.name === 'MongoError') {
                res.status(409).send({ message: err.message });
            }
            res.status(500).send({ message: `Error desconocido en el servidor.\n\n Error desconocido en el servidor cuando se actualizo factura con id='${id}'` });
        }

    });

    app.delete("/api/facturas/:id", async (req, res) => {

        const {id} = req.params;

        try {
            let factura = await Factura.findOneAndRemove({ _id: id });

            if (!factura) {
                return res.status(404).send({ message: 'factura no encontrada' });
            } else {
                return res.status(204).send({message:'la factura no se pudo borrar'}); 
            }
        } catch (err) {
            return res.status(500).send({ message: `la factura con id: '${id}' no pudo ser encontrada.` });
        }

    });














};
